/***************************************************************************
* Description     : LCD module export interface
* HW-Environment  : AVR MCU ATMega162-10MHz
*                   PCB eOscope v1.0
*                   XC9500 firmware v1.1
* SW-Environment  : AVR-GCC 3.4.3, Anjuta 1.2.1, AVR-INSIGHT, Avarice							
* Author          : Daniel Ciocea
* Revision        : 1.0 Initial version
****************************************************************************/

/* LCD Pins and Ports */
#define LCD_u8CtrlPort          PORTE            /* PORT for LCD control signals */
#define LCD_u8CtrlDdr           DDRE             /* DDR register of LCD control port */
#define LCD_nCtrlRW             PORTE2           /* pin for LCD Register Select */
#define LCD_nCtrlRS             PORTE1           /* pin for LCD Read/Write */
#define LCD_nCtrlE              PORTE0           /* pin for LCD Enable */

#define LCD_nCtrlRESET          PORTC3           /* pin for LCD Reset */
#define LCD_u8CtrlPortRESET     PORTC            /* port for LCD Reset */
#define LCD_u8CtrlDdrRESET      DDRC             /* DDR register of LCD Reset */
#define LCD_u8CtrlPinRESET      DDRC             /* PIN register of LCD Reset (usually not needed for RESET :)) */

#define LCD_u8DataPort          PORTA            /* PORT for LCD data signals */
#define LCD_u8DataDdr           DDRA             /* DDR register of LCD data port */
#define LCD_u8DataPin           PINA             /* PIN register of LCD data */
#define LCD_nBusyFlag           PINA7            /* LCD Busy Flag Pin */

/* LCD commands */
#define LCD_nSetLcdMode         0x00
#define LCD_nSetChPitch         0x01
#define LCD_nSetCharNum         0x02
#define LCD_nSetTimeDivs        0x03
#define LCD_nSetCurPos          0x04
#define LCD_nSetHomLAddr        0x08
#define LCD_nSetHomHAddr        0x09
#define LCD_nSetCurLAddr        0x0a
#define LCD_nSetCurHAddr        0x0b
#define LCD_nWriteData          0x0c
#define LCD_nReadData           0x0d
#define LCD_nClearBit           0x0e
#define LCD_nSetBit             0x0f

/* Export functions */
void LCD_vInitScreen (void);
void LCD_vClearScreen (void);
void LCD_vPrintSignalFrame (void);
void LCD_vPrintTrigger(uint8_t u8TrigLev);
void LCD_vPrintSignal (void);
void LCD_vPrintString (void);
void LCD_vPrintText (uint8_t Valoare);
void LCD_vPrintLogo (void);
